﻿// This is a "safe" class, meaning that it is created once 
// and never overwritten. Any custom code you add to this class 
// will be preserved when you regenerate your application.
//
// Typical customizations that may be done in this class include
//  - adding custom event handlers
//  - overriding base class methods

using System;
using System.Data.SqlTypes;
using BaseClasses;
using BaseClasses.Data;
using BaseClasses.Data.SqlProvider;

namespace V6MMYCS.Business
{

/// <summary>
/// Provides access to the schema information and record data of a database table (or view).
/// See <see cref="BaseOwnerTable"></see> for additional information.
/// </summary>
/// <remarks>
/// See <see cref="BaseOwnerTable"></see> for additional information.
/// <para>
/// This class is implemented using the Singleton design pattern.
/// </para>
/// </remarks>
/// <seealso cref="BaseOwnerTable"></seealso>
/// <seealso cref="BaseOwnerSqlTable"></seealso>
/// <seealso cref="OwnerSqlTable"></seealso>
/// <seealso cref="OwnerDefinition"></seealso>
/// <seealso cref="OwnerRecord"></seealso>
/// <seealso cref="BaseOwnerRecord"></seealso>
[SerializableAttribute()]
public class OwnerTable : BaseOwnerTable, System.Runtime.Serialization.ISerializable, ISingleton
{

#region "ISerializable Members"

    /// <summary>
    /// Overridden to use the <see cref="OwnerTable_SerializationHelper"></see> class 
    /// for deserialization of <see cref="OwnerTable"></see> data.
    /// </summary>
    /// <remarks>
    /// Since the <see cref="OwnerTable"></see> class is implemented using the Singleton design pattern, 
    /// this method must be overridden to prevent additional instances from being created during deserialization.
    /// </remarks>
    void System.Runtime.Serialization.ISerializable.GetObjectData(
        System.Runtime.Serialization.SerializationInfo info, 
        System.Runtime.Serialization.StreamingContext context)
    {
        info.SetType(typeof(OwnerTable_SerializationHelper)); //No other values need to be added
    }

#region "Class OwnerTable_SerializationHelper"

    [SerializableAttribute()]
    private class OwnerTable_SerializationHelper: System.Runtime.Serialization.IObjectReference
    {
        //Method called after this object is deserialized
        public virtual object GetRealObject(System.Runtime.Serialization.StreamingContext context)
        {
            return OwnerTable.Instance;
        }
    }

#endregion

#endregion

    /// <summary>
    /// References the only instance of the <see cref="OwnerTable"></see> class.
    /// </summary>
    /// <remarks>
    /// Since the <see cref="OwnerTable"></see> class is implemented using the Singleton design pattern, 
    /// this field is the only way to access an instance of the class.
    /// </remarks>
    public readonly static OwnerTable Instance = new OwnerTable();

    private OwnerTable()
    {
    }


} // End class OwnerTable

}
